# -*- coding: utf-8 -*-

from odoo.addons.http_routing.models.ir_http import slug as httpslug

from odoo.addons.http_routing.models import ir_http as httpRoutingObj
from odoo.addons.http_routing.models.ir_http import ModelConverter, RequestUID, unslug_url
# from odoo.addons.website.models.ir_http import
from odoo import api, fields, models, _
import odoo
import werkzeug
from odoo.http import request
import re
from odoo.exceptions import AccessError
from odoo.tools.safe_eval import safe_eval
import logging
from odoo.tools import ormcache
from werkzeug.exceptions import Forbidden, NotFound
_logger = logging.getLogger(__name__)

def slug(value):
    field = 'url_key'
    if field and isinstance(value, models.BaseModel) and hasattr(value, field):
        name = getattr(value, field)
        if name:
            suffix = getSuffix(value)
            if suffix:
                name = "{}{}".format(name, suffix)
            return name
    if not value:
        raise request.not_found()
    superslug = httpslug(value)
    return superslug

httpRoutingObj.slug = slug

def getSuffix(value):
    suffix = ''
    website = request.env['website'].get_current_website()
    use_suffix = website.use_suffix
    if website.manual_redirect:
        suffix  = website.manual_redirect
    elif use_suffix:
        suffix = request.env['website.rewrite'].sudo().getUrlSuffix(value)

    return suffix


class ModelConverterRewrite(ModelConverter):

    def __init__(self, url_map, model=False, domain='[]'):
        super(ModelConverter, self).__init__(url_map, model)
        self.domain = domain
        self.regex = r'(?:(\w{1,2}|\w[A-Za-z0-9-._]+?))(?=$|/)'

    def generate(self, uid, dom=None, args=None):
        Model = request.env[self.model].with_user(uid)
        # Allow to current_website_id directly in route domain
        args.update(current_website_id=request.env['website'].get_current_website().id)
        domain = safe_eval(self.domain, (args or {}).copy())
        if dom:
            domain += dom
        for record in Model.search(domain):
            yield record

    def to_url(self, value):
        try:
            return slug(value)
        except AccessError as e:
            pass
        return super().to_url(value)

    def to_python(self, value):
        value = self.unsetSuffix(value)
        _uid = RequestUID(value=value, converter=self)
        env = api.Environment(request.cr, _uid, request.context)
        record_id = None
        modelObj = self.getUrlKeyModel(value)
        self.model = modelObj
        field = 'url_key'
        if field and field in env[self.model]._fields:
            res = request.env[self.model].sudo().search([(field, '=', value)])
            if res:
                record_id = res[0].id
        if not record_id:
            customRegex = r'(?:(\w{1,2}|\w[A-Za-z0-9-_]+?\w)-)?(-?\d+)(?=$|/)'
            matching = re.match(customRegex, value)
            if matching:
                record_id = int(matching.group(2))
        if not record_id:
            return None
        if record_id < 0:
            if not env[self.model].browse(record_id).exists():
                record_id = abs(record_id)
        return env[self.model].browse(record_id)

    def getUrlKeyModel(self, value):
        model = self.model
        searchKey = "/"+value
        redirectObj = request.env['website.rewrite'].sudo()
        res = redirectObj.search([('url_to', '=', searchKey)], 0, 1, 'id desc')
        if not res:
            modelName = redirectObj.getSlugUrlKeyModel(value, model)
            if modelName:
                model = modelName
        if res:
            if res.rewrite_val != 'custom':
                model = res.rewrite_val
        return model

    def unsetSuffix(self, value):
        value = request.env['website.rewrite'].sudo().unsetUrlSuffix(value)
        return value


class IrHttp(models.AbstractModel):
    _inherit = 'ir.http'

    @classmethod
    def _get_converters(cls):
        return dict(
            super(IrHttp, cls)._get_converters(),
            model=ModelConverterRewrite,
        )

    @classmethod
    def _dispatch(cls):
        if not hasattr(request, "jsonrequest") and not hasattr(request, 'rerouting'):
            req_page = request.httprequest.path
            full_path = request.httprequest.full_path
            if "/web/database/" in req_page:
                return super(IrHttp, cls)._dispatch()
            redirectObj = request.env['website.rewrite'].sudo()
            if '/web/image' not in req_page and '/web/content' not in req_page:
                try:
                    page = ''
                    website_id = request.env['website'].get_current_website()
                    useServerRewrites = website_id.use_server_rewrites
                    useCategoryHierarchy = website_id.use_category_hierarchy
                    if useServerRewrites and req_page.count('/page')>0:
                        pass
                    # if 'page' in full_path:
                        # page = '/'.join(full_path.split('?')[-1].split('='))
                        # req_page = "/".join(full_path.split("page")[:-1])[:-1]
                    redirect = cls._serve_url_redirect(req_page, redirectObj)
                    if redirect and useServerRewrites and not useCategoryHierarchy:
                        suffix = redirectObj.getSuffix(redirect.rewrite_val)
                        url_to = redirect.url_to
                        if page:
                            url_to = url_to + "/" + page
                        if suffix:
                            url_to = "{}{}".format(url_to, suffix)
                        if redirect.rewrite_val == 'product.template' and website_id.use_category_url: # and not useCategoryHierarchy:
                            currentPage = request.httprequest.path
                            currentPage = currentPage.split('/')
                            currentPage = '/'.join(currentPage[:-1])
                            if currentPage:
                                url_to = currentPage+url_to
                        return werkzeug.utils.redirect(url_to)
                    else:
                        if '/shop/product' not in req_page and '/shop/category' not in req_page:
                            if useServerRewrites:
                                actualUrl = req_page
                                redirect = cls._serve_url_to_redirect(req_page, redirectObj)
                                if redirect:
                                    checkSuffix = cls._check_for_suffix(redirectObj, redirect, actualUrl)
                                    if checkSuffix:
                                        # return werkzeug.utils.redirect(checkSuffix, '302')
                                        req_page = checkSuffix
                                    frontEndLang = request.httprequest.cookies.get('frontend_lang')
                                    rewrite_val = redirect.rewrite_val
                                    defaultLang = ""
                                    defaultLangObj = website_id.default_lang_id
                                    if defaultLangObj:
                                        defaultLang = defaultLangObj.code
                                    if rewrite_val == 'product.template':
                                        website_id = request.env['website'].get_current_website()
                                        if not website_id.use_category_url and req_page.count('/')>1:
                                            return werkzeug.utils.redirect(checkSuffix, '302')
                                        if website_id.use_category_url and redirect.product_has_category and not req_page.count('/')>1:
                                            return werkzeug.utils.redirect(checkSuffix, '302')

                                        # check_valid_url = cls._check_valid_url(req_page,redirectObj)
                                        # if website_id.use_category_url and not cls._check_valid_url(req_page, redirectObj):
                                        #     return werkzeug.utils.redirect(False, '302')
                                        # if not cls._check_valid_url(req_page, redirectObj):
                                        #     return werkzeug.utils.redirect(False, '302')

                                        urlKey = '/'.join(req_page.split('/')[-1:])
                                        # urlKey = '/'.join(check_valid_url.get('final_url').split('/')[-1:])
                                        if frontEndLang != defaultLang:
                                            if frontEndLang:
                                                frontEndLang = frontEndLang.split("_")[0]
                                                redirectUrl = "/{}/shop/{}".format(frontEndLang, urlKey)
                                            else:
                                                redirectUrl = "/shop/" + urlKey
                                        else:
                                            redirectUrl = "/shop/" + urlKey
                                        return cls.reroute(redirectUrl)
                                    if rewrite_val == 'product.public.category':
                                        if not website_id.use_category_hierarchy and req_page.count('/')>1:
                                            return werkzeug.utils.redirect(checkSuffix, '302')
                                        if website_id.use_category_hierarchy and not redirect.is_category_parent and not req_page.count('/')>1:
                                            return werkzeug.utils.redirect(checkSuffix, '302')
                                        #
                                        # # check_valid_url = cls._check_valid_url(req_page, redirectObj)
                                        # if website_id.use_category_hierarchy and not cls._check_valid_url(req_page, redirectObj):
                                        #     return werkzeug.utils.redirect(False, '302')
                                        # if not cls._check_valid_url(req_page, redirectObj):
                                        #     return werkzeug.utils.redirect(False, '302')

                                        urlKey = '/'.join(req_page.split('/')[-1:])
                                        # urlKey = '/'.join(check_valid_url.get('final_url').split('/')[-1:])
                                        if frontEndLang != defaultLang:
                                            if frontEndLang:
                                                frontEndLang = frontEndLang.split("_")[0]
                                                redirectUrl = "/{}/shop/category/{}".format(frontEndLang, urlKey)
                                            else:
                                                redirectUrl = "/shop/category/" + urlKey
                                        else:
                                            redirectUrl = "/shop/category/" + urlKey
                                        if page:
                                            redirectUrl = redirectUrl + "/" + page
                                        return cls.reroute(redirectUrl)
                except Exception as e:
                    raise
                    _logger.info("^^^^^^^^^^^^Exception^^^^^^^^^^^^^^^^: %r", e)
                    pass
        res = super(IrHttp, cls)._dispatch()
        try:
            website_id = request.env['website'].get_current_website()
            useServerRewrites = website_id.use_server_rewrites
            if res and useServerRewrites:
                resData = res.get_data()
                if b"href='/shop/" in resData or b"href='/shop/category/" in resData or b'href="/shop/' in resData or b'href="/shop/category/' in resData:
                    currentPage = request.httprequest.environ.get('PATH_INFO')
                    currentPage = currentPage.replace("/shop/category/", "/")
                    redirectObj = request.env['website.rewrite'].sudo().get_parent_category(currentPage)
                    mainCat = redirectObj.get('mainCat')
                    use_server_rewrites = redirectObj.get('use_server_rewrites')
                    lang = redirectObj.get('lang')
                    dataString = resData.decode("utf-8")
                    if mainCat:
                        replaceUrl = "/shop/{}/".format(mainCat)
                        dataString = dataString.replace("/shop/", replaceUrl)
                    if lang:
                        replaceLangCatUrl = "/{}/shop/category/".format(lang)
                        replaceLangProUrl = "/{}/shop/".format(lang)
                        dataString = dataString.replace(replaceLangCatUrl, "/shop/category/").replace(replaceLangProUrl, "/shop/")
                    if use_server_rewrites:
                        dataString = dataString.replace("/shop/category/", "/")
                    res.set_data(dataString.encode("utf-8"))
        except Exception as e:
            pass
        return res
    #
    # @classmethod
    # def _check_valid_url(cls,req_page, redirectObj):
    #     is_valid_url = False
    #     # url_keys_to_remove = []
    #     # final_url = req_page
    #     categ_prod_url = req_page.split('/')
    #     for rec in categ_prod_url:
    #         if rec:
    #             url_to = '/%s' % rec.split('.')[0]
    #             is_active_url = redirectObj.search([('url_to', '=', url_to)],limit=1)
    #             if not is_active_url:
    #                 website_id =  request.env['website'].get_current_website()
    #                 for red in website_id.manual_redirects_ids:
    #                     if red.manual_redirect == url_to:
    #                         is_valid_url = True
    #                         # final_url.replace('/%s'%url_to,'')
    #                         break
    #                     else:
    #                         is_valid_url = False
    #                 if not is_valid_url:
    #                     break
    #             else:
    #                 is_valid_url = True
    #     return is_valid_url
    #     # return {"is_valid_url":is_valid_url,"final_url":final_url}

    @classmethod
    def _serve_url_redirect(cls, req_page, redirectObj):
        req_page = redirectObj.unsetUrlSuffix(req_page)
        urlKey = ''.join(req_page.split('/')[-1:])
        if not urlKey:
            if req_page[-1:] == '/':
                urlKey = req_page.replace('/', '')
        domain = [('url_from', '=', req_page)]
        if urlKey:
            urlKey = "/" + urlKey
            # urlKey = "/seo/" + urlKey
            domain = ['|', ('url_from', '=', req_page),('url_from', '=', urlKey)]
        redirectObjs = redirectObj.search(domain, limit=1,order='id desc')
        if redirectObjs:
            if redirectObjs.url_to == req_page or redirectObjs.url_to == urlKey:
                return False
        return redirectObjs

    @classmethod
    # @ormcache('req_page', 'redirectObj')
    def _serve_url_to_redirect(cls, req_page, redirectObj):
        try:
            req_page = redirectObj.unsetUrlSuffix(req_page)
            urlKey = ''.join(req_page.split('/')[-1:])
            if not urlKey:
                if req_page[-1:] == '/':
                    urlKey = req_page.replace('/', '')
            domain = [('url_to', '=', req_page)]
            transDomain = ['|', ('value', '=', req_page.replace('/', '')),('value', '=', urlKey)]
            if urlKey:
                urlKey = "/" + urlKey
                domain = ['|', ('url_to', '=', req_page),('url_to', '=', urlKey)]
            redirectObjs = redirectObj.search(domain,order='id desc', limit=1)
            if not redirectObjs:
                resObjs = request.env['ir.translation'].sudo().search(transDomain, limit=1)
                if resObjs:
                    urlKey = "/" + resObjs.src
                    domain = [('url_to', '=', urlKey)]
                    redirectObjs = redirectObj.search(domain,order='id desc', limit=1)
        except Exception as e:
            _logger.info("------Exception--_serve_url_to_redirect----- : %r", e)
        return redirectObjs

    @classmethod
    def _check_for_suffix(cls, redirectObj, redirectObjs, actualUrl):
        suffixData = redirectObj.trackSuffix(redirectObjs.rewrite_val)
        seoSuffix = suffixData.get('suffix')

        if suffixData.get('use_suffix') == False and seoSuffix:
            if seoSuffix not in actualUrl:
                actualUrl = "{}{}".format(actualUrl, seoSuffix)
                return actualUrl
        elif suffixData.get('use_suffix') and seoSuffix:
            if seoSuffix not in actualUrl:
                actualUrl = "{}{}".format(actualUrl, seoSuffix)
                return actualUrl
        else:
            if seoSuffix and seoSuffix in actualUrl:
                actualUrl = actualUrl.replace(seoSuffix, '')
                return actualUrl
        return False

class IrUiView(models.Model):
    _inherit = ["ir.ui.view"]

    @api.model
    def _prepare_qcontext(self):
        qcontext = super(IrUiView, self)._prepare_qcontext()
        qcontext['slug'] = slug
        qcontext['unslug_url'] = unslug_url
        return qcontext
