# -*- coding: utf-8 -*-

from odoo import api, fields, models, _
import logging

_logger = logging.getLogger(__name__)


class ProductTemplate(models.Model):

    _inherit = "product.template"

    url_key = fields.Char(string="URL Key")

    @api.model
    def create(self, vals):
        res = super(ProductTemplate, self).create(vals)
        if res.url_key in ['', False, None]:
            self.env['website.rewrite'].setSeoUrlKey('pattern_product', res)
        return res

    def write(self, vals):
        for proObj in self:
            if vals.get('url_key'):
                vals = self.env['website.rewrite'].createRedirectForRewrite(vals, proObj, 'product.template', 'pattern_category')
        res = super(ProductTemplate, self).write(vals)
        return res

    def update_seo_url(self):
        productIds = self._context.get('active_ids')
        productObjs = self.search([('id', 'in', productIds)])
        resp = self.env['website.rewrite'].setSeoUrlKey('pattern_product', productObjs)
        text = "SEO Url key of {} product(s) have been successfully updated.".format(len(productObjs))
        if resp:
            failedIds = ", ".join(resp)
            updatedProducts = len(productObjs) - len(resp)
            text = "Products with internal reference [{}] are failed to update. Reason : SEO URL Key must be unique!".format(failedIds)
            if updatedProducts:
                text = "SEO Url key of {} product(s) have been successfully updated and products with internal reference [{}] are failed to update. Reason : SEO URL Key must be unique!".format(updatedProducts, failedIds)

        return self.env['sh.message.wizard'].genrated_message(text)


    def open_website_url(self):
        res = super(ProductTemplate, self).open_website_url()
        website_id = self.env["website"].get_current_website()

        if website_id.use_server_rewrites and res.get("url"):
            url = self.url_key
            if website_id.use_suffix:
                url += website_id.suffix_product
            res.update({"url": url})
        return res

    def _compute_website_url(self):
        website_id = self.env["website"].get_current_website()
        if website_id:
            for product in self:
                if product._origin:
                    website_id.get_seo_product_url(product)
        # else:
        #     super(ProductTemplate, self)._compute_website_url()

class ProductProduct(models.Model):
    _inherit = 'product.product'

    @api.depends_context('lang')
    @api.depends('product_tmpl_id.website_url', 'product_template_attribute_value_ids')
    def _compute_product_website_url(self):
        for product in self:
            product.website_url = "%s"%(product.product_tmpl_id.website_url)
